----------------------------------------------------------------------------------
-- Company:        CERN, BE-CO 
-- Engineer:       Carlos Gil Soriano
-- 
-- Create Date:    01:22:27 07/26/2012 
-- Design Name:    m25p32 top_tp level
-- Module Name:    m25p32_top_tp - RTL 
-- Project Name:   Level Conversion Circuits
-- Target Devices: Spartan 6
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------

library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.m25p32_pkg.ALL;
use work.spi_master_pkg.ALL;
use work.m25p32_top_tb_pkg.ALL;
use work.spi_analyser_pkg.ALL;

entity  m25p32_top_tb is
   end  m25p32_top_tb;

architecture Behavioral of m25p32_top_tb is 


   signal wb_rst_i            : STD_LOGIC;
   signal s_rst_spi_analyser  : STD_LOGIC;
   signal wb_clk              : STD_LOGIC;

   signal s_wb_we         : STD_LOGIC;
   signal s_wb_stb        : STD_LOGIC;
   signal s_wb_cyc        : STD_LOGIC;
   signal s_wb_sel        : STD_LOGIC_VECTOR (3  downto 0);
   signal s_wb_data_i     : STD_LOGIC_VECTOR (31 downto 0); 
   signal s_wb_data_o     : STD_LOGIC_VECTOR (31 downto 0); 
   signal s_wb_addr       : STD_LOGIC_VECTOR (c_WORDS_PER_PAGE_BITS downto 0);

   signal s_wb_ack        : STD_LOGIC; 
   signal s_wb_rty        : STD_LOGIC; 
   signal s_wb_err        : STD_LOGIC;

   signal s_spi_mosi      : STD_LOGIC;
   signal s_spi_miso      : STD_LOGIC := '0';
   signal s_spi_clk       : STD_LOGIC;
   signal s_spi_cs_n      : STD_LOGIC;

   signal s_FMI           : r_FMI;
   signal s_SR_m25p32     : r_SR_m25p32;

   --! Signals for the spi_analyser
   signal s_SPI0          : r_SPI0; 
   signal s_SPI0_slv      : STD_LOGIC_VECTOR(31 downto 0);
   signal s_SPI1          : r_SPI1; 
   signal s_SPI1_slv      : STD_LOGIC_VECTOR(31 downto 0);
   signal s_SPI2          : r_SPI2; 
   signal s_SPI2_slv      : STD_LOGIC_VECTOR(15 downto 0);

   signal s_inst_check    : STD_LOGIC_VECTOR (7    downto 0) := (others => '0');
   signal s_addr_check    : STD_LOGIC_VECTOR (23   downto 0) := (others => '0');
   signal s_data_check    : STD_LOGIC_VECTOR (2047 downto 0) := (others => '0');

   signal s_end_inst_flag : STD_LOGIC; 
   signal s_end_addr_flag : STD_LOGIC; 
   signal s_end_data_flag : STD_LOGIC;

   signal s_page          : r_page;
begin



   --! Connections for the m25p32 
   uut: m25p32_top port map(
      wb_rst_i       =>  wb_rst_i, 
      wb_clk         =>  wb_clk, 

      wb_we_i        =>  s_wb_we, 
      wb_stb_i       =>  s_wb_stb, 
      wb_cyc_i       =>  s_wb_cyc, 
      wb_sel_i       =>  s_wb_sel, 
      wb_data_i      =>  s_wb_data_i, 
      wb_data_o      =>  s_wb_data_o, 
      wb_addr_i      =>  s_wb_addr,

      wb_ack_o       =>  s_wb_ack, 
      wb_rty_o       =>  s_wb_rty, 
      wb_err_o       =>  s_wb_err, 

      prom_mosi_o    =>  s_spi_mosi, 
      prom_cclk_o    =>  s_spi_clk, 
      prom_cs0_b_n_o =>  s_spi_cs_n,
      prom_din_i     =>  s_spi_miso);

   tester : spi_analyser port map(
          rst_i            => s_rst_spi_analyser, 
          SPI0_i           => s_SPI0, 
          SPI1_i           => s_SPI1, 

          spi_mosi_o       => s_spi_mosi, 
          spi_miso_i       => s_spi_miso, 
          spi_clk_o        => s_spi_clk, 
          spi_cs_n_o       => s_spi_cs_n, 
                                             
          inst_check_o     => s_inst_check, 
          addr_check_o     => s_addr_check, 
          data_check_o     => s_data_check,
                                             
          end_inst_flag_o  => s_end_inst_flag, 
          end_addr_flag_o  => s_end_addr_flag, 
          end_data_flag_o  => s_end_data_flag);


   --! @brief Process that generates the wb_clk
   clk_i_process : process
   begin
      wb_clk <= '0';
        wait for work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD/2;
      wb_clk <= '1';
        wait for work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD/2;
   end process;


   --! @brief Testbench
   p_simulation : process

      --! @brief Init procedure for the test
      procedure init_cond is
      begin
         --! We reset the wishbone bus
            wait until rising_edge(wb_clk);
         s_wb_we       <= '0'; 
         s_wb_stb      <= '0'; 
         s_wb_cyc      <= '0'; 
         s_wb_sel      <= (others => '0'); 
         s_wb_data_i   <= (others => '0'); 
         s_wb_addr     <= (others => '0'); 
         --! And also the miso line
         s_spi_miso    <= '0';
      end procedure;

      procedure wishbone_write_SR (reg_data : r_SR_m25p32) is
         variable v_reg_data  : STD_LOGIC_VECTOR(31 downto 0); 
      begin
         v_reg_data( 7 downto 0) :=  f_STD_LOGIC_VECTOR(reg_data);
         v_reg_data(31 downto 8) :=  (others => '0');
         wishbone_write(wb_clk, s_wb_we,
                        s_wb_stb, s_wb_cyc, s_wb_sel,
                        s_wb_data_i, s_wb_addr,         
                        v_reg_data, c_SR_m25p32_addr);
      end procedure;

      procedure send_operation (
             operation : t_operations;
             sector: NATURAL;
             page  : NATURAL) is
         variable v_FMI : r_FMI;
      begin
         v_FMI.OPR  := '1'; 
         v_FMI.x    := (others => '0'); 
         v_FMI.OP   := operation; 
         v_FMI.PG   := to_unsigned(page,   c_PAGES_PER_SECTOR_BITS); 
         v_FMI.SCT  := to_unsigned(sector, c_SECTOR_BITS); 
         v_FMI.y    := (others => '0'); 
         wishbone_write(wb_clk, s_wb_we,
                        s_wb_stb, s_wb_cyc, s_wb_sel,
                        s_wb_data_i, s_wb_addr,         
                        X"00" & f_STD_LOGIC_VECTOR(v_FMI), c_FMI_addr);
      end procedure;

      procedure wishbone_write_word_in_page(word : r_word;
                                         pos  : NATURAL) is
         variable v_word: STD_LOGIC_VECTOR(31 downto 0);
         variable v_pos : UNSIGNED (c_WORDS_PER_PAGE_BITS downto 0);
      begin
         v_word := f_STD_LOGIC_VECTOR(word); 
         v_pos  := '1' & to_unsigned(pos,c_WORDS_PER_PAGE_BITS); 
         wishbone_write(wb_clk, s_wb_we,
                        s_wb_stb, s_wb_cyc, s_wb_sel,
                        s_wb_data_i, s_wb_addr,         
                        v_word, STD_LOGIC_VECTOR(v_pos));
      end procedure;

      procedure read_SPI_rcved_data(signal read_data :
                                       out STD_LOGIC_VECTOR(31 downto 0)) is
      begin
         wishbone_read (wb_clk, s_wb_we,
                        s_wb_stb, s_wb_cyc, s_wb_sel,
                        s_wb_data_o, s_wb_addr, read_data, 
                        c_DATA_READ_addr);
      end procedure;

   begin

      --! This line is cosmetic because this register is not loaded in the
      --! core until a call to send_instruction procedure is issued.
      s_FMI    <= c_FMI_default;
      --! First we perform the reset to the module
      init_cond;
      wb_rst_i  <= '1';
        wait for work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD*10;
      wb_rst_i  <= '0';
        wait for work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD*10;

      -------------------------------------------------------------------------------
      --! Test  Target
      --!    0    WRSR
      --!    1      PP   
      --!    2      SE   
      --!    3      BE   
      --!    4    RDSR 
      --!    5    RDID 
      --!    6    READ 
      -------------------------------------------------------------------------------

      s_SR_m25p32 <= (WIP  => '0', 
                      WEL  => '0', 
                      BP0  => '1', 
                      BP1  => '1', 
                      BP2  => '1', 
                      x    => (others => '0'), 
                      SRWD => '0');

         wait until rising_edge(wb_clk);

      --! Some quick tests
      --! [0] Testing WRSR
      wishbone_write_SR(s_SR_m25p32);
      send_operation(WRSR, 0, 0);
      --! Know it should accept the instruction
         wait for 512*work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD;

      --! [1] Testing PP
      s_page <= fill_page;
         wait until rising_edge(wb_clk);

      for i in 0 to c_PAGE_SIZE/4 - 1 loop
         wishbone_write_word_in_page(s_page(i), i);
      end loop;
         wait until rising_edge(wb_clk);
      send_operation(PP, 0, 0);
         wait for 51200*work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD;
         wait until rising_edge(wb_clk);

      --! [2] Testing SE
      send_operation(SE, 7, 0);
         wait for 512*work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD;
         wait until rising_edge(wb_clk);

      --! [3] Testing BE
      send_operation(BE, 0, 0);
         wait for 256*work.m25p32_top_tb_pkg.c_WISHBONE_PERIOD;
         wait until rising_edge(wb_clk);

      assert false report "No error. Simulation ends." severity failure;
   end process p_simulation;

end;
